# -*- coding: utf-8 -*-
import unittest

import mock


class TestPyramidFacebook(unittest.TestCase):

    def test_includeme(self):
        settings = {
            'facebook.namespace': 'namespace',
            'facebook.secret_key': 'secret_key'
            }

        config = mock.Mock()
        config.registry.settings = settings

        # TEST includeme
        from pyramid_facebook import includeme
        self.assertIsNone(includeme(config))

        self.assertEqual(6, config.include.call_count)
        prefix = '/namespace'

        self.assertEqual(
            mock.call('pyramid_facebook.lib', route_prefix=prefix),
            config.include.call_args_list[0]
            )

        self.assertEqual(
            mock.call('pyramid_facebook.security', route_prefix=prefix),
            config.include.call_args_list[1]
            )
        self.assertEqual(
            mock.call('pyramid_facebook.auth', route_prefix=prefix),
            config.include.call_args_list[2]
            )

        self.assertEqual(
            mock.call('pyramid_facebook.canvas', route_prefix=prefix),
            config.include.call_args_list[3]
            )

        self.assertEqual(
            mock.call('pyramid_facebook.credits', route_prefix=prefix),
            config.include.call_args_list[4]
            )

        self.assertEqual(
            mock.call('pyramid_facebook.real_time', route_prefix=prefix),
            config.include.call_args_list[5]
            )

    def test_includeme_exception(self):
        from pyramid_facebook import includeme
        bad_settings = {}

        config = mock.MagicMock()
        config.registry.settings = bad_settings

        self.assertRaises(KeyError, includeme, config)

        bad_settings = {
            'facebook.secret_key': 'secret_key'
        }

        config.registry.settings = bad_settings
        self.assertRaises(KeyError, includeme, config)



# -*- coding: utf-8 -*-
import unittest

import mock

from pyramid_facebook.tests import conf


class TestFacebookCanvas(unittest.TestCase):

    def test_includeme(self):
        from pyramid_facebook.canvas import includeme
        config = mock.Mock()

        self.assertIsNone(includeme(config))

        self.assertEqual(1, config.add_route.call_count)

        expected = mock.call(
            'facebook_canvas',
            '/',
            request_method='POST',
            request_param='signed_request',
            factory='pyramid_facebook.security.SignedRequestContext',
            )

        self.assertEqual(expected, config.add_route.call_args_list[0])

        config.scan.assert_called_once_with(package='pyramid_facebook.canvas')

    def test_prompt_authorize(self):
        from pyramid_facebook.canvas import prompt_authorize

        settings = conf

        request = mock.MagicMock()
        request.scheme = 'http'
        request.GET = {'fetchez': 'la vache', 'code': '342435634blab'}
        request.route_path.return_value = '/facebook/oauth'
        request.registry.settings = settings

        result = prompt_authorize(request.context, request)

        request.route_path.assert_called_once_with('facebook_canvas_oauth')

        expected = """200 OK
Content-Type: text/html; charset=UTF-8
Content-Length: 244

<html>
  <body>
    <script>
      window.top.location = "https://www.facebook.com/dialog/oauth/?client_id=1234567890&redirect_uri=http%3A%2F%2Fapps.facebook.com%2Ffacebook%2Foauth%3Ffetchez%3Dla%2Bvache&scope=";
    </script>
  </body>
</html>"""
        self.assertEqual(expected, str(result))

    def test_canvas(self):
        from pyramid_facebook.canvas import canvas
        request = mock.MagicMock()
        with self.assertRaises(NotImplementedError):
            canvas(request.context, request)


class TestFacebookCanvasDecorator(unittest.TestCase):

    def test_init(self):
        from pyramid_facebook.canvas import facebook_canvas
        from pyramid_facebook.security import ViewCanvas

        dec = facebook_canvas(**{})

        self.assertEqual(ViewCanvas, dec.permission)
        self.assertEqual('facebook_canvas', dec.route_name)


class TestOnCanvaSubscriber(unittest.TestCase):

    def test_on_canvas(self):
        from pyramid_facebook.canvas import on_canvas
        from pyramid_facebook.events import CanvasRequested

        event = mock.Mock()

        on_canvas(event)

        self.assertEqual(1, event.request.registry.notify.call_count)

        param = event.request.registry.notify.call_args_list[0][0][0]

        self.assertIsNotNone(param)

        self.assertIsInstance(param, CanvasRequested)

        self.assertEqual(param.context, event.request.context)
        self.assertEqual(param.request, event.request)



# -*- coding: utf-8 -*-
from pyramid_facebook.lib import encrypt_signed_request

__all__ = ['conf', 'get_signed_request']

conf = {
    'facebook.app_id': '1234567890',
    'facebook.secret_key': 'abcdef1234567890',
    'facebook.namespace': 'test_canvas',
    'facebook.scope': '',
    'pyramid.debug_routematch': True,
    }


def get_signed_request(**params):
    return encrypt_signed_request(conf['facebook.secret_key'], params)



# -*- coding: utf-8 -*-
import json
import unittest

from pyramid.request import Request


class TestChangeNotification(unittest.TestCase):

    def test_json_body(self):
        from pyramid_facebook.events import ChangeNotification

        obj = dict(a=123, b=4321)
        request = Request(dict(), body=json.dumps(obj))

        cn = ChangeNotification(None, request)

        self.assertIsInstance(cn.json_body, dict)
        self.assertEqual(obj, cn.json_body)



# -*- coding: utf-8 -*-
import unittest

from facepy import FacepyError
import mock
from pyramid.httpexceptions import HTTPOk, HTTPForbidden


class TestRealTimeSubscriptions(unittest.TestCase):

    @mock.patch('pyramid_facebook.real_time.nor_predicate')
    @mock.patch('pyramid_facebook.real_time.headers_predicate')
    @mock.patch('pyramid_facebook.real_time.request_params_predicate')
    def test_includeme(self, m_req_predicate, m_header_predicate, m_nor_predicate):
        from pyramid_facebook.real_time import includeme
        config = mock.Mock()

        self.assertIsNone(includeme(config))

        self.assertEqual(5, config.add_route.call_count)

        expected = mock.call(
            'facebook_real_time_subscription_verification',
            '/real-time',
            request_method='GET',
            custom_predicates=[
                m_req_predicate.return_value,
                ]
            )
        self.assertEqual(expected, config.add_route.call_args_list[0])

        expected = mock.call(
            'facebook_real_time_notification',
            '/real-time',
            request_method='POST',
            factory='pyramid_facebook.security.RealTimeNotificationContext',
            custom_predicates=[m_header_predicate.return_value]
            )
        self.assertEqual(expected, config.add_route.call_args_list[1])

        expected = mock.call(
            'facebook_real_time_subscriptions',
            '/real-time/subscriptions',
            request_method='GET',
            request_param='access_token',
            factory='pyramid_facebook.security.AdminContext',
            )
        self.assertEqual(expected, config.add_route.call_args_list[2])

        expected = mock.call(
            'facebook_real_time_subscriptions_update',
            '/real-time/subscriptions',
            request_method='POST',
            custom_predicates=[
                m_nor_predicate.return_value,
                m_req_predicate.return_value
                ],
            factory='pyramid_facebook.security.AdminContext',
            )
        self.assertEqual(expected, config.add_route.call_args_list[3])

        expected = mock.call(
            'facebook_real_time_subscriptions_delete',
            '/real-time/subscriptions',
            request_method='DELETE',
            request_param='access_token',
            factory='pyramid_facebook.security.AdminContext',
            )
        self.assertEqual(expected, config.add_route.call_args_list[4])

        self.assertEqual(2, m_req_predicate.call_count)

        self.assertEqual(
            mock.call('hub.challenge','hub.verify_token',**{'hub.mode': 'subscribe'}),
            m_req_predicate.call_args_list[0]
            )
        self.assertEqual(
            mock.call('fields', 'access_token'),
            m_req_predicate.call_args_list[1]
            )

        m_header_predicate.assert_called_once_with(
            'X-Hub-Signature',
            **{'Content-Type': 'application/json'}
            )

        m_nor_predicate.assert_called_once_with(object=('user', 'permissions', 'page', 'errors'))


    @mock.patch('pyramid_facebook.real_time.get_application_access_token')
    def test_verify_real_time_subscription(self, m_req):
        from pyramid_facebook.real_time import verify_real_time_subscription

        m_req.return_value = 'token'

        request = mock.MagicMock()
        request.params = {
            'hub.verify_token':'token',
            'hub.challenge': 'challenge'
            }

        result = verify_real_time_subscription(request)

        self.assertEqual(request.params['hub.challenge'], result.body)

    @mock.patch('pyramid_facebook.real_time.get_application_access_token')
    def test_verify_real_time_subscription_wrong_token(self, m_req):
        from pyramid_facebook.real_time import verify_real_time_subscription

        m_req.return_value = 'not the same token'

        request = mock.MagicMock()
        request.params = {
            'hub.verify_token':'token',
            'hub.challenge': 'challenge'
            }

        self.assertRaises(HTTPForbidden, verify_real_time_subscription, request)


    @mock.patch('pyramid_facebook.real_time.get_application_access_token')
    def test_verify_real_time_subscription_facepy_error(self, m_req):
        from pyramid_facebook.real_time import verify_real_time_subscription

        m_req.side_effect = FacepyError('shit happens.')

        request = mock.MagicMock()
        request.params = {
            'hub.verify_token':'token',
            'hub.challenge': 'challenge'
            }

        self.assertRaises(HTTPForbidden, verify_real_time_subscription, request)


    @mock.patch('pyramid_facebook.real_time.GraphAPI')
    @mock.patch('pyramid_facebook.real_time.get_application_access_token')
    def test_list_real_time_subscriptions(self, m_get_token, m_graph_api):
        from pyramid_facebook.real_time import list_real_time_subscriptions

        request = mock.MagicMock()

        result = list_real_time_subscriptions(request)

        self.assertEqual(m_graph_api.return_value.get.return_value, result)

        m_get_token.assert_called_once_with(
            request.registry.settings.__getitem__.return_value,
            request.registry.settings.__getitem__.return_value,
            )

        self.assertEqual(2, request.registry.settings.__getitem__.call_count)
        self.assertEqual(
            mock.call('facebook.app_id'),
            request.registry.settings.__getitem__.call_args_list[0]
            )
        self.assertEqual(
            mock.call('facebook.secret_key'),
            request.registry.settings.__getitem__.call_args_list[1]
            )

        m_graph_api.assert_called_once_with(m_get_token.return_value)
        m_graph_api.return_value.get.assert_called_once_with(
            '%s/subscriptions' % request.registry.settings.__getitem__.return_value,
            )

    @mock.patch('pyramid_facebook.real_time.GraphAPI')
    @mock.patch('pyramid_facebook.real_time.get_application_access_token')
    def test_update_real_time_subscriptions(self, m_get_token, m_graph_api):
        from pyramid_facebook.real_time import update_real_time_subscription

        request = mock.MagicMock()

        result = update_real_time_subscription(request)

        self.assertEqual(m_graph_api.return_value.post.return_value, result)

        m_get_token.assert_called_once_with(
            request.registry.settings.__getitem__.return_value,
            request.registry.settings.__getitem__.return_value,
            )

        self.assertEqual(3, request.registry.settings.__getitem__.call_count)
        self.assertEqual(
            mock.call('facebook.app_id'),
            request.registry.settings.__getitem__.call_args_list[0]
            )
        self.assertEqual(
            mock.call('facebook.secret_key'),
            request.registry.settings.__getitem__.call_args_list[1]
            )
        self.assertEqual(
            mock.call('facebook.app_id'),
            request.registry.settings.__getitem__.call_args_list[2]
            )

        request.route_url.assert_called_once_with(
            'facebook_real_time_subscription_verification'
            )

        m_graph_api.assert_called_once_with(m_get_token.return_value)

        m_graph_api.return_value.post.assert_called_once_with(
            '%s/subscriptions' % request.registry.settings.__getitem__.return_value,
            object=request.params.__getitem__.return_value,
            fields=request.params.__getitem__.return_value,
            callback_url=request.route_url.return_value,
            verify_token=m_get_token.return_value,
            )

    @mock.patch('pyramid_facebook.real_time.GraphAPI')
    @mock.patch('pyramid_facebook.real_time.get_application_access_token')
    def test_delete_real_time_subscriptions(self, m_get_token, m_graph_api):
        from pyramid_facebook.real_time import delete_real_time_subscription

        request = mock.MagicMock()

        delete_real_time_subscription(request)

        request.params.get.assert_called_once_with('object')

        self.assertEqual(
            mock.call('facebook.app_id'),
            request.registry.settings.__getitem__.call_args_list[0]
            )
        self.assertEqual(
            mock.call('facebook.secret_key'),
            request.registry.settings.__getitem__.call_args_list[1]
            )
        self.assertEqual(
            mock.call('facebook.app_id'),
            request.registry.settings.__getitem__.call_args_list[2]
            )

        m_get_token.assert_called_once_with(
            request.registry.settings.__getitem__.return_value,
            request.registry.settings.__getitem__.return_value,
            )

        m_graph_api.assert_called_once_with(m_get_token.return_value)

        m_graph_api.return_value.delete.assert_called_once_with(
            '%s/subscriptions' % request.registry.settings.__getitem__.return_value,
            object=request.params.get.return_value,
            )

    def test_render_facepy_error(self):
        from pyramid_facebook.real_time import render_facepy_error
        exc = FacepyError('error message')
        exc.code = 190

        result = render_facepy_error(exc, None)

        self.assertEqual(
            dict(error=dict(type='FacepyError', code=190, message='error message')),
            result
            )

    @mock.patch('pyramid_facebook.real_time.ChangeNotification')
    def test_process_real_time_notification(self, m_change):
        from pyramid_facebook.real_time import process_real_time_notification
        request = mock.Mock()

        result = process_real_time_notification(request.context, request)

        self.assertIsInstance(result, HTTPOk)

        request.registry.notify.assert_called_once_with(m_change.return_value)

        m_change.assert_called_once_with(request.context, request)



# -*- coding: utf-8 -*-
import urllib
import logging

from pyramid.httpexceptions import HTTPForbidden
from pyramid.response import Response
from pyramid.view import view_config, view_defaults

from pyramid_facebook.events import OauthAccept, OauthDeny
from pyramid_facebook.lib import Base, js_redirect_tpl
from pyramid_facebook.security import Authenticate


log = logging.getLogger(__name__)


def includeme(config):
    log.debug('Adding route "facebook_canvas_oauth".')
    config.add_route(
        'facebook_canvas_oauth',
        '/oauth',
        request_method='POST',
        request_param='signed_request',
        factory='pyramid_facebook.security.SignedRequestContext',
        )
    config.scan(package='pyramid_facebook.auth')


@view_defaults(route_name='facebook_canvas_oauth')
class FacebookCanvasOAuth(Base):

    def _redirect_to_canvas(self):
        log.debug('redirect to canvas')
        path = self.request.route_path('facebook_canvas')
        query_string = self.request.GET.copy()
        query_string.pop('code', None)
        p = {'location': "//apps.facebook.com%s?%s" % (
            path,
            # we could filter out "error*" fields
            urllib.urlencode(query_string),
            )}
        return Response(js_redirect_tpl % p)

    @view_config(permission=Authenticate)
    def oauth_accept(self):
        log.debug('user accepts')
        try:
            self.request.registry.notify(OauthAccept(self.context, self.request))
        except Exception:
            log.exception(
                'Oauth accept with context=%r, params=%r.',
                self.context,
                self.request.params
                )
        return self._redirect_to_canvas()

    @view_config(context=HTTPForbidden, request_param="error")
    def oauth_deny(self):
        log.debug('user denies')
        try:
            self.request.registry.notify(OauthDeny(self.context, self.request))
        except Exception:
            log.exception(
                'Oauth deny with context=%r, params=%r: %s',
                self.context,
                self.request.params
                )
        return self._redirect_to_canvas()



# -*- coding: utf-8 -*-
import unittest
import json

import mock


from pyramid import testing
from pyramid.config import Configurator
from pyramid.security import Allow

from pyramid_facebook.security import SignedRequestContext
from pyramid_facebook.tests import conf

TEST_USER = 123


def _get_signed_request(user_id=None):
    from pyramid_facebook.lib import encrypt_signed_request
    return encrypt_signed_request(
        conf['facebook.secret_key'],
        {
            'user_id': user_id if user_id else TEST_USER,
            'user': {
                'country': 'ca'
                }
            }
        )


def _get_request(signed_request=None):
    request = mock.MagicMock()
    signed_request = (signed_request
                      if signed_request else _get_signed_request())
    request.params = {'signed_request': signed_request}
    request.registry.settings = conf.copy()
    return request


class TestIncludeme(unittest.TestCase):

    @mock.patch('pyramid_facebook.security.ACLAuthorizationPolicy')
    @mock.patch('pyramid_facebook.security.FacebookAuthenticationPolicy')
    def test_includeme(self, FacebookAuthenticationPolicy,
                       ACLAuthorizationPolicy):
        from pyramid_facebook.security import includeme

        config = mock.MagicMock(spec=Configurator)

        includeme(config)

        config.set_authentication_policy.assert_called_once_with(
            FacebookAuthenticationPolicy.return_value
            )
        config.set_authorization_policy.assert_called_once_with(
            ACLAuthorizationPolicy.return_value
            )
        FacebookAuthenticationPolicy.assert_called_once_with()
        ACLAuthorizationPolicy.assert_called_once_with()


class TestFacebookCreditsContext(unittest.TestCase):

    @mock.patch('pyramid_facebook.security.json')
    def test_order_details(self, m_json):
        from pyramid_facebook.security import FacebookCreditsContext
        ctx = FacebookCreditsContext(_get_request())
        ctx.facebook_data = data = mock.MagicMock()

        self.assertEqual(m_json.loads.return_value, ctx.order_details)
        m_json.loads.assert_called_once_with(data['credits']['order_details'])

    def test_earned_currency_data(self):
        from pyramid_facebook.security import FacebookCreditsContext
        ctx = FacebookCreditsContext(_get_request())
        ctx.facebook_data = {
            'credits': {
                'order_details': json.dumps({
                    'items': [
                        {'data': '{"modified": "earned_currency_data"}'}
                        ]
                    })
                }
            }
        self.assertEqual(
            'earned_currency_data',
            ctx.earned_currency_data
            )

    def test_earned_currency_data_invalid_json(self):
        from pyramid_facebook.security import FacebookCreditsContext
        ctx = FacebookCreditsContext(_get_request())
        # with Invalid json data:
        ctx.facebook_data = {
            'credits': {
                'order_details': json.dumps({
                    'items': [
                        {'data': '{INVALID JSON "}'}
                        ]
                    })
                }
            }
        self.assertEqual(
            None,
            ctx.earned_currency_data
            )

    def test_order_info(self):
        from pyramid_facebook.security import FacebookCreditsContext
        ctx = FacebookCreditsContext(_get_request())
        ctx.facebook_data = mock.MagicMock()

        self.assertEqual(ctx.facebook_data["credits"]["order_info"],
                         ctx.order_info)

    def test_item(self):
        from pyramid_facebook.security import FacebookCreditsContext
        ctx = FacebookCreditsContext(_get_request())
        ctx.facebook_data = mock.MagicMock()
        t = json.dumps({'items': [{'title': 'an item'}]})
        ctx.facebook_data.__getitem__.return_value.__getitem__.return_value = t

        self.assertEqual(
            {'title': 'an item'},
            ctx.item,
            )


class TestSignedRequestContext(unittest.TestCase):

    def test_init(self):
        from pyramid_facebook.security import (
            SignedRequestContext,
            ViewCanvas,
            Authenticate,
            FacebookUser,
            RegisteredUser,
            )

        self.assertEqual(
            [
                (Allow, FacebookUser, Authenticate),
                (Allow, RegisteredUser, ViewCanvas),
                ],
            SignedRequestContext.__acl__
            )

        SignedRequestContext(None)

    def test_facebook_data_not_set(self):
        from pyramid_facebook.security import SignedRequestContext
        sr = SignedRequestContext(None)
        self.assertEqual(None, sr.facebook_data)

    def test_facebook_data_property(self):
        from pyramid_facebook.security import SignedRequestContext
        sr = SignedRequestContext(None)
        sr.facebook_data = 'something'
        self.assertEqual('something', sr.facebook_data)

    def test_repr(self):
        from pyramid_facebook.security import SignedRequestContext

        ctx = SignedRequestContext(_get_request())
        ctx.facebook_data = 'something'

        self.assertEqual(
            "<SignedRequestContext facebook_data='something'>",
            ctx.__repr__()
            )

    def test_user(self):
        from pyramid_facebook.security import SignedRequestContext
        sr = SignedRequestContext(None)
        sr.facebook_data = {'user': 12345}
        self.assertEqual(12345, sr.user)

    def test_country(self):
        from pyramid_facebook.security import SignedRequestContext
        sr = SignedRequestContext(None)
        sr.facebook_data = {'user': {'country': 'FR'}}
        self.assertEqual('FR', sr.user_country)


class TestAccessTokenContext(unittest.TestCase):

    def test_access_token(self):
        from pyramid_facebook.security import AccessTokenContext
        request = mock.Mock()
        atc = AccessTokenContext(request)
        atc.user_dict = {u'k': 123}
        atc.user_id = 1000

        self.assertEqual(atc.request, request)
        self.assertEqual({u'k': 123}, atc.user_dict)
        self.assertEqual(1000, atc.user_id)


class TestRealTimeNotificationContext(unittest.TestCase):

    def test_init(self):
        from pyramid_facebook.security import (
            RealTimeNotificationContext,
            Allow,
            XHubSigned,
            NotifyRealTimeChanges
            )

        request = mock.Mock()

        rtnc = RealTimeNotificationContext(request)

        self.assertEqual(request, rtnc.request)
        self.assertEqual(
            ((Allow, XHubSigned, NotifyRealTimeChanges), ),
            rtnc.__acl__
            )


class TestFacebookAuthenticationPolicy(unittest.TestCase):

    def setUp(self):
        self.config = testing.setUp()

    def tearDown(self):
        testing.tearDown()

    def _get_request(self, context_cls=SignedRequestContext):
        request = testing.DummyRequest()
        request.params = {'signed_request': mock.MagicMock(spec=str)}
        request.registry.settings = {
            'facebook.secret_key': 'a secret',
            }
        request.context = mock.MagicMock(spec=context_cls)
        return request

    @mock.patch('pyramid_facebook.security.SignedRequest')
    def test_parse_signed_request(self, SignedRequest):
        from pyramid_facebook.security import FacebookAuthenticationPolicy
        request = self._get_request()
        SignedRequest.parse.return_value = {u'user_id': 123}
        fap = FacebookAuthenticationPolicy()
        result = fap._parse_signed_request(request)

        SignedRequest.parse.assert_called_once_with(
            request.params['signed_request'],
            request.registry.settings['facebook.secret_key'],
            )
        self.assertEqual(123, result)

    def test_unauthenticated_userid(self):
        from pyramid_facebook.security import FacebookAuthenticationPolicy
        request = self._get_request()
        fap = FacebookAuthenticationPolicy()
        self.assertIsNone(fap.unauthenticated_userid(request))

    def test_unauthenticated_userid_key_error(self):
        from pyramid_facebook.security import FacebookAuthenticationPolicy
        request = self._get_request()
        request.context = 'Whatever which causes a KeyError'
        fap = FacebookAuthenticationPolicy()
        self.assertIsNone(fap.unauthenticated_userid(request))

    @mock.patch('pyramid_facebook.security.SignedRequest')
    def test_effective_principals(self, SignedRequest):
        from pyramid_facebook.security import (
            FacebookAuthenticationPolicy,
            RegisteredUser,
            FacebookUser,
            )
        request = self._get_request()
        SignedRequest.parse.return_value = {u'user_id': 123}
        fap = FacebookAuthenticationPolicy()
        self.assertEqual(
            ['system.Everyone', 'system.Authenticated', 123, RegisteredUser,
             FacebookUser],
            fap.effective_principals(request)
            )

    def test_effective_principals_attribute_error(self):
        from pyramid_facebook.security import FacebookAuthenticationPolicy
        request = self._get_request()
        fap = FacebookAuthenticationPolicy()
        self.assertEqual(['system.Everyone'],
                         fap.effective_principals(request))

    def test_effective_principals_key_error(self):
        from pyramid_facebook.security import FacebookAuthenticationPolicy
        request = self._get_request(
            context_cls=type('UnknownContext', (object, ), {})
            )
        fap = FacebookAuthenticationPolicy()

        # Key Error handling:
        self.assertEqual(['system.Everyone'],
                         fap.effective_principals(request))

    def test_remember(self):
        from pyramid_facebook.security import FacebookAuthenticationPolicy
        request = self._get_request()

        fap = FacebookAuthenticationPolicy()

        principal = "principal"
        kwargs = {"a": "a_val", "b": "b_val"}

        # FacebookAuthenticationPolicy does not support a remember/forget
        # mecanism, user is authenticated on each request
        self.assertListEqual([], fap.remember(request, principal, **kwargs))

    def test_forget(self):
        from pyramid_facebook.security import FacebookAuthenticationPolicy
        request = self._get_request()

        fap = FacebookAuthenticationPolicy()

        # FacebookAuthenticationPolicy does not support a remember/forget
        # mecanism, user is authenticated on each request
        self.assertListEqual([], fap.forget(request))

    def test_parse_signed_request_with_no_signed_request(self):
        from pyramid_facebook.security import FacebookAuthenticationPolicy
        request = self._get_request()
        del request.params['signed_request']

        fap = FacebookAuthenticationPolicy()

        self.assertIsNone(fap._parse_signed_request(request))

    @mock.patch('pyramid_facebook.security.SignedRequest')
    def test_parse_signed_request_with_no_user_id(self, SignedRequest):
        from pyramid_facebook.security import FacebookAuthenticationPolicy
        request = self._get_request()
        SignedRequest.parse.return_value = {'no_user_id': ':-o'}

        fap = FacebookAuthenticationPolicy()

        self.assertIsNone(fap._parse_signed_request(request))

    @mock.patch('pyramid_facebook.security.SignedRequest')
    def test_parse_signed_request_with_invalid_user_id(self, SignedRequest):
        from pyramid_facebook.security import FacebookAuthenticationPolicy
        request = self._get_request()
        SignedRequest.parse.return_value = {u'user_id': u'not an int'}

        fap = FacebookAuthenticationPolicy()

        self.assertIsNone(fap._parse_signed_request(request))

    @mock.patch('pyramid_facebook.security.GraphAPI')
    def test_validate_access_token_with_no_access_token(self, GraphAPI):
        from pyramid_facebook.security import FacebookAuthenticationPolicy
        request = self._get_request()

        fap = FacebookAuthenticationPolicy()

        self.assertIsNone(fap._validate_access_token(request))

    @mock.patch('pyramid_facebook.security.GraphAPI')
    def test_validate_access_token_with_facepy_error(self, GraphAPI):
        from pyramid_facebook.security import (
            FacebookAuthenticationPolicy,
            FacepyError,
            AccessTokenContext,
            )
        request = self._get_request(context_cls=AccessTokenContext)
        request.context.user_dict = {}
        request.params = {u'access_token': u'123qwerty'}

        api = GraphAPI.return_value = mock.Mock()
        api.get.side_effect = FacepyError('oooops')

        fap = FacebookAuthenticationPolicy()

        self.assertIsNone(fap._validate_access_token(request))

    @mock.patch('pyramid_facebook.security.GraphAPI')
    def test_validate_access_token_malformed_fb_response(self, GraphAPI):
        from pyramid_facebook.security import (
            FacebookAuthenticationPolicy,
            AccessTokenContext,
            )
        request = self._get_request(context_cls=AccessTokenContext)
        request.params = {u'access_token': u'123qwerty'}

        api = GraphAPI.return_value = mock.Mock()
        api.get.return_value = {u'id': u'not an int'}

        fap = FacebookAuthenticationPolicy()

        self.assertEqual(None, fap._validate_access_token(request))

    @mock.patch('pyramid_facebook.security.GraphAPI')
    def test_validate_access_token(self, GraphAPI):
        from pyramid_facebook.security import (
            FacebookAuthenticationPolicy,
            AccessTokenContext,
            )
        request = self._get_request(context_cls=AccessTokenContext)
        request.params = {u'access_token': u'123qwerty'}

        api = GraphAPI.return_value = mock.Mock()
        api.get.return_value = {u'id': u'12345'}

        fap = FacebookAuthenticationPolicy()

        self.assertEqual(12345, fap._validate_access_token(request))

    def test_get_principals_from_access_token(self):
        from pyramid_facebook.security import (
            FacebookAuthenticationPolicy,
            AccessTokenContext,
            FacebookUser
            )
        request = self._get_request(context_cls=AccessTokenContext)
        request.context.user_id = 1234

        fap = FacebookAuthenticationPolicy()

        self.assertEqual(
            [FacebookUser],
            fap._get_principals_from_access_token(request)
            )

    @mock.patch('pyramid_facebook.security.GraphAPI')
    def test_get_principals_from_admin_context_with_facepy_error(self,
                                                                 GraphAPI):
        from pyramid_facebook.security import (
            FacebookAuthenticationPolicy,
            AdminContext,
            FacepyError,
            )
        request = self._get_request(context_cls=AdminContext)
        request.params = {u'access_token': u'123qwerty'}

        api = GraphAPI.return_value = mock.Mock()
        api.get.side_effect = FacepyError('opps')

        fap = FacebookAuthenticationPolicy()

        self.assertEqual(
            ['facebook-user'],
            fap._get_principals_from_admin_context(request)
            )

    @mock.patch('pyramid_facebook.security.GraphAPI')
    def test_get_principals_from_admin_context(self, GraphAPI):
        from pyramid_facebook.security import (
            FacebookAuthenticationPolicy,
            AdminContext,
            )
        request = self._get_request(context_cls=AdminContext)
        request.params = {u'access_token': u'123qwerty'}

        api = GraphAPI.return_value = mock.Mock()
        api.get.return_value = {u'data': [{u'id': u'123'}, {u'id': u'234'}]}

        fap = FacebookAuthenticationPolicy()

        self.assertEqual(
            ['facebook-user', u'123', u'234'],
            fap._get_principals_from_admin_context(request)
            )

    @mock.patch('pyramid_facebook.security.hmac')
    def test_get_principals_from_signature_wrong_signature(self, hmac):
        from pyramid_facebook.security import (
            FacebookAuthenticationPolicy,
            RealTimeNotificationContext,
            )
        request = self._get_request(context_cls=RealTimeNotificationContext)
        request.headers[u'X-Hub-Signature'] = u'sha1=signature'

        hmac.new.return_value.hexdigest.return_value = u'WRONG'

        fap = FacebookAuthenticationPolicy()

        self.assertEqual(
            [],
            fap._get_principals_from_signature(request)
            )

    @mock.patch('pyramid_facebook.security.hmac')
    def test_get_principals_from_signature(self, hmac):
        from pyramid_facebook.security import (
            FacebookAuthenticationPolicy,
            RealTimeNotificationContext,
            XHubSigned,
            )
        request = self._get_request(context_cls=RealTimeNotificationContext)
        request.headers[u'X-Hub-Signature'] = u'sha1=signature'

        hmac.new.return_value.hexdigest.return_value = u'signature'

        fap = FacebookAuthenticationPolicy()

        self.assertEqual(
            [XHubSigned],
            fap._get_principals_from_signature(request)
            )



# -*- coding: utf-8 -*-
import unittest

import mock

from pyramid.response import Response

from pyramid_facebook.tests import conf


class TestAuth(unittest.TestCase):

    def test_includeme(self):
        from pyramid_facebook.auth import includeme
        config = mock.Mock()

        self.assertIsNone(includeme(config))

        self.assertEqual(1, config.add_route.call_count)

        expected = mock.call(
            'facebook_canvas_oauth',
            '/oauth',
            request_method='POST',
            request_param='signed_request',
            factory='pyramid_facebook.security.SignedRequestContext',
            )

        self.assertEqual(expected, config.add_route.call_args_list[0])

        config.scan.assert_called_once_with(package='pyramid_facebook.auth')


class TestFacebookOauth(unittest.TestCase):

    maxDiff = None

    def test_init(self):
        from pyramid_facebook.auth import FacebookCanvasOAuth
        request = mock.MagicMock()
        canvas = FacebookCanvasOAuth(request.context, request)
        self.assertEqual(request, canvas.request)
        self.assertEqual(request.context, canvas.context)

    def test_oauth_accept(self):
        from pyramid_facebook.auth import FacebookCanvasOAuth
        from pyramid_facebook.events import OauthAccept
        from pyramid_facebook.lib import js_redirect_tpl

        settings = conf

        request = mock.MagicMock()
        request.scheme = 'http'
        request.GET = {'spam': 'ham', 'code': '2342342345gibberish'}
        request.route_path.return_value = '/facebook'
        request.registry.settings = settings

        view = FacebookCanvasOAuth(request.context, request)

        result = view.oauth_accept()

        request.route_path.assert_called_once_with('facebook_canvas')
        self.assertTrue(request.registry.notify.called)

        self.assertIsInstance(
            request.registry.notify.call_args[0][0],
            OauthAccept
            )

        expected = str(Response(
            js_redirect_tpl %
            {'location': "//apps.facebook.com/facebook?spam=ham"}
            ))

        self.assertEqual(expected, str(result))

    def test_oauth_deny(self):
        from pyramid_facebook.auth import FacebookCanvasOAuth
        from pyramid_facebook.events import OauthDeny
        from pyramid_facebook.lib import js_redirect_tpl

        settings = conf

        request = mock.MagicMock()
        request.scheme = 'http'
        request.GET = {'spam': 'ham'}
        request.route_path.return_value = '/facebook'
        request.registry.settings = settings

        view = FacebookCanvasOAuth(request.context, request)

        result = view.oauth_deny()

        request.route_path.assert_called_once_with('facebook_canvas')
        self.assertTrue(request.registry.notify.called)

        self.assertIsInstance(
            request.registry.notify.call_args[0][0],
            OauthDeny
            )
        expected = str(Response(
            js_redirect_tpl %
            {'location': "//apps.facebook.com/facebook?spam=ham"}
            ))

        self.assertEqual(expected, str(result))

    @mock.patch('pyramid_facebook.auth.log')
    def test_oauth_exception(self, m_log):
        from pyramid_facebook.auth import FacebookCanvasOAuth

        request = mock.MagicMock()
        request.registry.notify.side_effect = Exception('boooom!')

        view = FacebookCanvasOAuth(request.context, request)
        view.oauth_accept()
        self.assertTrue(m_log.exception.called)

        m_log.error.reset_mock()
        view.oauth_deny()
        self.assertTrue(m_log.exception.called)



# -*- coding: utf-8 -*-
import logging
import functools

from textwrap import dedent

import venusian
from pyramid.view import view_config
from pyramid.httpexceptions import HTTPOk
from pyramid.response import Response

from pyramid_facebook.lib import request_params_predicate
from pyramid_facebook.events import (
    DisputedOrder,
    EarnedCurrencyOrder,
    PlacedItemOrder,
    RefundedOrder,
    )
from pyramid_facebook.security import ViewCanvas

log = logging.getLogger(__name__)


def includeme(config):
    """Adds routes related to facebook credits.

    Routes Added:

    * ``facebook_payments_get_items`` associated to url ``/{namespace}/credits``.

    * ``facebook_payments_status_update_placed`` associated to url
      ``/{namespace}/credits``.

    * ``facebook_payments_status_update_disputed`` associated to url
      ``/{namespace}/credits``.

    * ``facebook_payments_status_update_refunded`` associated to url
      ``/{namespace}/credits``.
    """
    log.debug('Adding route "facebook_payments_get_items".')
    config.add_route(
        'facebook_payments_get_items',
        '/credits',
        request_method='POST',
        custom_predicates=[
            request_params_predicate(
                'signed_request',
                'buyer',
                'receiver',
                'order_id',
                'order_info',
                method='payments_get_items',
                )
            ],
        factory='pyramid_facebook.security.FacebookCreditsContext',
        )

    update_predicate = request_params_predicate(
        'signed_request',
        'order_details',
        'order_id',
        method='payments_status_update',
        )

    log.debug('Adding route "facebook_payments_status_update_placed".')
    config.add_route(
        'facebook_payments_status_update_placed',
        '/credits',
        request_method='POST',
        custom_predicates=[
            update_predicate,
            request_params_predicate(status='placed'),
            ],
        factory='pyramid_facebook.security.FacebookCreditsContext',
        )

    log.debug('Adding route "facebook_payments_status_update_disputed".')
    config.add_route(
        'facebook_payments_status_update_disputed',
        '/credits',
        request_method='POST',
        custom_predicates=[
            update_predicate,
            request_params_predicate(status='disputed'),
            ],
        factory='pyramid_facebook.security.FacebookCreditsContext',
        )

    log.debug('Adding route "facebook_payments_status_update_refunded".')
    config.add_route(
        'facebook_payments_status_update_refunded',
        '/credits',
        request_method='POST',
        custom_predicates=[
            update_predicate,
            request_params_predicate(status='refunded'),
            ],
        factory='pyramid_facebook.security.FacebookCreditsContext',
        )

    config.scan(package='pyramid_facebook.credits')


class facebook_payments_get_items(object):
    """Decorator to register the function to process facebook credits
    `payments_get_items <http://developers.facebook.com/docs/credits/callback/#payments_get_items>`_.

    Decorated function receives 2 positional parameters:

    * ``context``: The :class:`~pyramid_facebook.security.FacebookCreditsContext`
      the request is associated with. ``context.facebook_data["user"]`` gives
      information about user's locale which would permit to return different
      languages.

    * ``request``: The request itself.

    It is possible to access `order_info` via :attr:`context.order_info
    <pyramid_facebook.security.FacebookCreditsContext.order_info>`:

    Decorated function must return a dictionary structured as::

        {
            # Required:
            "title":       "100 diamonds",
            "description": "100 shiny diamonds!",
            "price":       1000,
            "image_url":   "http://hadriendavid.com/images/100dimonds.png",

            # Optional (according to facebook doc):
            "item_id": "123",
            "data":    "whatever"
        }

    Example::

        @facebook_payments_get_items()
        def get_item(context, request):
            ...
            return {
                "title": a_title,
                "description": a_description,
                "price": a_price_in_facebook_credits,
                "image_url": an_image_url
                }
    """
    def __call__(self, wrapped):
        # as documented on http://docs.pylonsproject.org/projects/pyramid/en/1.3-branch/narr/hooks.html#registering-configuration-decorators
        venusian.attach(wrapped, self._register)

        log.debug('facebook_payments_get_items provided by %s', wrapped)
        return wrapped

    def _register(self, scanner, name, wrapped):
        """Register view to pyramid framework.
        """
        config = scanner.config
        config.add_view(
            view=self._decorate(wrapped),
            permission=ViewCanvas,
            route_name='facebook_payments_get_items',
            renderer='json',
            )
        log.debug('Adding views %s for route facebook_payments_get_items', wrapped)

    def _validate(self, item_content):
        "Validate item content"
        if not isinstance(item_content, dict):
            raise TypeError('dict expected, received %s' % type(item_content))
        expected = {
            'title'      : (str, unicode,),
            'description': (str, unicode,),
            'price'      : (int,         ),
            'image_url'  : (str, unicode,),
            }
        for k, v in expected.iteritems():
            if type(item_content.get(k)) not in v:
                raise TypeError('Expected %r, received %r' % (
                    expected,
                    item_content
                    ))
        # title must not be empty
        if len(item_content['title']) < 1:
            raise ValueError('item title must not be an empty string')
        return item_content

    def _decorate(self, wrapped):
        @functools.wraps(wrapped)
        def wrapper(context, request):
            log.debug(
                'facebook_payments_get_items call with facebook_data=%s',
                context.facebook_data
                )
            result = self._validate(wrapped(context, request))
            return {
                "content": [result],
                "method": "payments_get_items"
                }
        return wrapper

class facebook_get_currency(object):
    """Decorator factory for app currency OpenGraph objects.

    Application currencies need to be registered to enable `in-app currency
    offers`__, i.e. an "earn coins" feature.  This factory returns a
    decorator that creates one route and registers the decorated function
    as view.

    .. __: https://developers.facebook.com/docs/credits/offers/#app_currency_offers

    This factory exposes the route name (``'facebook_get_currency'``)
    and the OpenGraph object URI (``/currencies/<name>``).  The decorated function
    receives 2 positional parameters, context and request, and must return a
    dictionary with this structure::

       {
           'app_id': Facebook application ID,
           'name': currency name, displayed as is in all languages,
           'description': currency description,
           'image_url': URL to 50x50px image,
           'rate': currency rate,
       }

    The currency name must be a plural form, and will be displayed as-is
    in all languages. The currency rate is used to compute the amount of
    currency gained.  See the Facebook documentation link above for more
    information.

    The OpenGraph object must be registered into the OpenGraph before using
    an earn currency feature.  You can use the Facebook OpenGraph explorer
    to do it.
    """

    def __call__(self, wrapped):
        venusian.attach(wrapped, self._register)
        log.info('facebook_get_currency provided by %s', wrapped)
        return wrapped

    def _register(self, scanner, name, wrapped):
        # create route and view
        route_name = 'facebook_get_currency'
        pattern = '/currencies/{name}'
        log.debug('Adding route %s and view %s', route_name, wrapped)
        scanner.config.add_route(name=route_name,
                                 pattern=pattern,
                                 request_method='GET',
                                )
        scanner.config.add_view(view=self._decorate(wrapped),
                                route_name=route_name,
                               )

    def _decorate(self, wrapped):
        @functools.wraps(wrapped)
        def wrapper(context, request):
            log.debug(
                'facebook_get_currency called with context=%s',
                context)
            variables = wrapped(context, request)
            if not isinstance(variables, dict):
                raise TypeError('expected dict, got %r' % type(variables))
            # missing or extra keys will cause TypeError in % operation below
            # TODO make image_url optional
            variables['url'] = request.current_route_url()

            template = dedent("""\
                <head prefix="og: http://ogp.me/ns#
                          fb: http://ogp.me/ns/fb#
                          fbpayment: http://ogp.me/ns/fb/fbpayment#">
                    <meta property="fb:app_id" content="%(app_id)s">
                    <meta property="og:type" content="fbpayment:currency">
                    <meta property="og:url" content="%(url)s">
                    <meta property="og:title" content="%(name)s">
                    <meta property="og:description" content="%(description)s">
                    <meta property="og:image" content="%(image_url)s">
                    <meta property="fbpayment:rate" content="%(rate)s">
                </head>
                """)
            return Response(template % variables)
        return wrapper


@view_config(route_name='facebook_payments_status_update_disputed')
def facebook_payments_status_update_disputed(context, request):
    try:
        log.debug('Order Status Update Disputed')
        request.registry.notify(DisputedOrder(context, request))
    except Exception:
        log.exception(
            'facebook_payments_status_update_disputed with context=%r, '
            'request.params=%r',
            context,
            request,
            )
    return HTTPOk()


@view_config(route_name='facebook_payments_status_update_refunded')
def facebook_payments_status_update_refunded(context, request):
    try:
        log.debug('Order Status Update Refunded')
        request.registry.notify(RefundedOrder(context, request))
    except Exception:
        log.exception(
            'facebook_payments_status_update_refunded with context=%r, '
            'request.params=%r.',
            context,
            request,
            )
    return HTTPOk()


@view_config(
    route_name='facebook_payments_status_update_placed',
    permission=ViewCanvas,
    renderer='json'
    )
def facebook_payments_status_update_placed(context, request):
    if context.earned_currency_data:
        log.debug('Order Status Update - Earned App Currency')
        event_class = EarnedCurrencyOrder
    else:
        log.debug('Order Status Update - Placed Item Order')
        event_class = PlacedItemOrder
    try:
        request.registry.notify(event_class(context, request))
    except Exception:
        log.exception(
            'facebook_payments_status_update_placed with context=%r, '
            'request.params=%r.',
            context,
            request,
            )
        status = 'canceled'
    else:
        log.debug(
            'facebook_payments_status_update_placed settled %r',
            context.order_details,
            )
        status = 'settled'
    return {
        'content': {
            'status': status,
            'order_id': context.order_details['order_id'],
            },
        'method': 'payments_status_update',
        }



# -*- coding: utf-8 -*-
import logging

from facepy import FacepyError, get_application_access_token, GraphAPI

from pyramid.httpexceptions import HTTPForbidden, HTTPOk, HTTPInternalServerError
from pyramid.response import Response
from pyramid.view import view_config

from pyramid_facebook.events import ChangeNotification
from pyramid_facebook.lib import (
    headers_predicate,
    request_params_predicate,
    nor_predicate,
    )
from pyramid_facebook.security import UpdateSubscription, NotifyRealTimeChanges

log = logging.getLogger(__name__)


def includeme(config):
    """Adds routes related to real time updates.

    Routes Added:

    * ``facebook_real_time_subscription_verification`` associated to url
        /{namespace}/real-time

    * ``facebook_real_time_notification`` associated to url
        /{namespace}/real-time

    * ``facebook_real_time_subscriptions`` associated to url
        /{namespace}/real-time/subscriptions

    * ``facebook_real_time_subscriptions_update`` associated to url
        /{namespace}/real-time/subscriptions

    * ``facebook_real_time_subscriptions_delete`` associated to url
        /{namespace}/real-time/subscriptions
    """
    log.debug('Adding route "facebook_real_time_subscription_verification".')
    config.add_route(
        'facebook_real_time_subscription_verification',
        '/real-time',
        request_method='GET',
        custom_predicates=[
            request_params_predicate(
                'hub.challenge',
                'hub.verify_token',
                **{'hub.mode': 'subscribe'}
                ),
            ]
        )

    log.debug('Adding route "facebook_real_time_notification".')
    config.add_route(
        'facebook_real_time_notification',
        '/real-time',
        request_method='POST',
        factory='pyramid_facebook.security.RealTimeNotificationContext',
        custom_predicates=[
            headers_predicate(
                'X-Hub-Signature',
                **{'Content-Type': 'application/json'}
                )
            ]
        )

    log.debug('Adding route "facebook_real_time_subscriptions".')
    config.add_route(
        'facebook_real_time_subscriptions',
        '/real-time/subscriptions',
        request_method='GET',
        request_param='access_token',
        factory='pyramid_facebook.security.AdminContext',
        )

    log.debug('Adding route "facebook_real_time_subscriptions_update".')
    config.add_route(
        'facebook_real_time_subscriptions_update',
        '/real-time/subscriptions',
        request_method='POST',
        custom_predicates=[
            nor_predicate(object=('user', 'permissions', 'page', 'errors')),
            request_params_predicate('fields', 'access_token'),
            ],
        factory='pyramid_facebook.security.AdminContext',
        )

    log.debug('Adding route "facebook_real_time_subscriptions_delete".')
    config.add_route(
        'facebook_real_time_subscriptions_delete',
        '/real-time/subscriptions',
        request_method='DELETE',
        request_param='access_token',
        factory='pyramid_facebook.security.AdminContext',
        )

    config.scan(package='pyramid_facebook.real_time')


@view_config(route_name='facebook_real_time_subscription_verification')
def verify_real_time_subscription(request):
    verify_token = request.params['hub.verify_token']
    settings = request.registry.settings
    try:
        access_token = get_application_access_token(
            settings['facebook.app_id'],
            settings['facebook.secret_key'],
            )
        if access_token == verify_token:
            return Response(request.params['hub.challenge'])
    except FacepyError:
        log.exception('get_application_access_token failed')
    raise HTTPForbidden()


@view_config(context=FacepyError, renderer='json')
def render_facepy_error(exc, request):
    return dict(
        error=dict(
            type=exc.__class__.__name__,
            code=getattr(exc, 'code'),
            message=exc.message,
            )
        )


@view_config(
    route_name='facebook_real_time_subscriptions',
    renderer='json',
    permission=UpdateSubscription,
    )
def list_real_time_subscriptions(request):
    settings = request.registry.settings
    app_id = settings['facebook.app_id']
    secret_key = settings['facebook.secret_key']
    access_token = get_application_access_token(app_id, secret_key)
    return GraphAPI(access_token).get('%s/subscriptions' % app_id)


@view_config(
    route_name='facebook_real_time_subscriptions_update',
    renderer='json',
    permission=UpdateSubscription,
    )
def update_real_time_subscription(request):
    settings = request.registry.settings
    fb_object = request.params['object']
    fields = request.params['fields']
    access_token = get_application_access_token(
        settings['facebook.app_id'],
        settings['facebook.secret_key'],
        )
    # note that url of route named `facebook_real_time_subscription_verification`
    # is the same as route named `facebook_real_time_notification` but with
    # different predicates
    url = request.route_url('facebook_real_time_subscription_verification')
    return (GraphAPI(access_token).post(
        '%s/subscriptions' % settings['facebook.app_id'],
        object=fb_object,
        fields=fields,
        callback_url=url,
        verify_token=access_token
        ))


@view_config(
    route_name='facebook_real_time_subscriptions_delete',
    renderer='json',
    permission=UpdateSubscription,
    )
def delete_real_time_subscription(request):
    settings = request.registry.settings
    fb_object = request.params.get('object')
    access_token = get_application_access_token(
        settings['facebook.app_id'],
        settings['facebook.secret_key'],
        )
    return (GraphAPI(access_token).delete(
        '%s/subscriptions' % settings['facebook.app_id'],
        object=fb_object
        ))


@view_config(
    route_name='facebook_real_time_notification',
    permission=NotifyRealTimeChanges,
    )
def process_real_time_notification(context, request):
    try:
        request.registry.notify(ChangeNotification(context, request))
    except:
        log.exception(
            'process_real_time_notification - json_body: %s',
            request.json_body
            )
        # raise an HTTP error so facebook will send back same update later
        raise HTTPInternalServerError()
    return HTTPOk()



# -*- coding: utf-8 -*-
import unittest

import mock

from pyramid.httpexceptions import HTTPOk


class TestFacebookPaymentsGetItemsDecorator(unittest.TestCase):

    @mock.patch('pyramid_facebook.credits.request_params_predicate')
    def test_includeme(self, m_predicate):
        from pyramid_facebook.credits import includeme

        config = mock.Mock()

        self.assertIsNone(includeme(config))

        self.assertEqual(4, config.add_route.call_count)

        expected = mock.call(
            'facebook_payments_get_items',
            '/credits',
            request_method='POST',
            custom_predicates=[
                m_predicate.return_value
                ],
            factory='pyramid_facebook.security.FacebookCreditsContext',
            )
        self.assertEqual(expected, config.add_route.call_args_list[0])

        expected = mock.call(
            'facebook_payments_status_update_placed',
            '/credits',
            request_method='POST',
            custom_predicates=[
                m_predicate.return_value,
                m_predicate.return_value,
                ],
            factory='pyramid_facebook.security.FacebookCreditsContext',
            )
        self.assertEqual(expected, config.add_route.call_args_list[1])

        expected = mock.call(
            'facebook_payments_status_update_disputed',
            '/credits',
            request_method='POST',
            custom_predicates=[
                m_predicate.return_value,
                m_predicate.return_value,
                ],
            factory='pyramid_facebook.security.FacebookCreditsContext',
            )
        self.assertEqual(expected, config.add_route.call_args_list[2])

        expected = mock.call(
        'facebook_payments_status_update_refunded',
        '/credits',
        request_method='POST',
        custom_predicates=[
                m_predicate.return_value,
                m_predicate.return_value,
            ],
        factory='pyramid_facebook.security.FacebookCreditsContext',
        )
        self.assertEqual(expected, config.add_route.call_args_list[3])

        config.scan.assert_called_once_with(package='pyramid_facebook.credits')

        self.assertEqual(5, m_predicate.call_count)

        expected = mock.call(
            'signed_request',
            'buyer',
            'receiver',
            'order_id',
            'order_info',
            method='payments_get_items',
            )
        self.assertEqual(expected, m_predicate.call_args_list[0])

        expected = mock.call(
            'signed_request',
            'order_details',
            'order_id',
            method='payments_status_update',
            )
        self.assertEqual(expected, m_predicate.call_args_list[1])

        self.assertEqual(mock.call(status='placed'), m_predicate.call_args_list[2])
        self.assertEqual(mock.call(status='disputed'), m_predicate.call_args_list[3])
        self.assertEqual(mock.call(status='refunded'), m_predicate.call_args_list[4])

    @mock.patch('pyramid_facebook.credits.venusian')
    def test_call(self, m_venusian):
        from pyramid_facebook.credits import facebook_payments_get_items
        wrapped = mock.Mock()

        dec = facebook_payments_get_items()
        res = dec(wrapped)

        m_venusian.attach.assert_called_once_with(wrapped, dec._register)
        self.assertEqual(wrapped, res)

    def test_register(self):
        from pyramid_facebook.security import ViewCanvas
        from pyramid_facebook.credits import facebook_payments_get_items
        wrapped = mock.Mock()
        scanner = mock.Mock()
        scanner.config = config = mock.Mock()

        dec = facebook_payments_get_items()
        dec._decorate = mock.Mock()

        dec._register(scanner, 'name', wrapped)

        config.add_view.assert_called_once_with(
            view=dec._decorate.return_value,
            permission=ViewCanvas,
            route_name='facebook_payments_get_items',
            renderer='json',
            )

    def test_validate(self):
        from pyramid_facebook.credits import facebook_payments_get_items

        dec = facebook_payments_get_items()

        self.assertRaises(TypeError, dec._validate, ['not_a_dict'])

        self.assertRaises(TypeError, dec._validate, {'not': 'good_keys'})

        bad_type = {
            'title'      : u"A title",
            'description': u"A description",
            'price'      : 1.2,  # not an int
            'image_url'  : "can be a wrong url :-(",
            }
        self.assertRaises(TypeError, dec._validate, bad_type)

        bad_title = {
            'title'      : u"", # empty title
            'description': u"A description",
            'price'      : 1,
            'image_url'  : "can be a wrong url :-(",
            }
        self.assertRaises(ValueError, dec._validate, bad_title)

        good_content =  {
            'title'      : u"A title",
            'description': u"A description",
            'price'      : 100,
            'image_url'  : u"any str or unicode",
            }

        self.assertEqual(good_content, dec._validate(good_content))

    def test_decorate(self):
        from pyramid_facebook.credits import facebook_payments_get_items
        context = mock.MagicMock()
        request = mock.Mock()
        # set function attributes needed by functools.wraps
        wrapped = mock.Mock()
        wrapped.__name__ = 'somefunction'

        dec = facebook_payments_get_items()
        dec._validate = mock.Mock()

        wrapper = dec._decorate(wrapped)

        # test wrapper call:
        res = wrapper(context, request)

        # assertions:
        dec._validate.assert_called_once_with(wrapped.return_value)
        wrapped.assert_called_once_with(
            context,
            request
            )
        self.assertEqual(res, {
            "content": [dec._validate.return_value],
            "method": "payments_get_items"
            })


class TestFacebookPaymentsCallback(unittest.TestCase):

    @mock.patch('pyramid_facebook.credits.DisputedOrder')
    def test_facebook_payments_status_update_disputed(self, m_order):
        from pyramid_facebook.credits import facebook_payments_status_update_disputed
        ctx = mock.Mock()
        req = mock.Mock()

        res = facebook_payments_status_update_disputed(ctx, req)

        req.registry.notify.assert_called_once_with(m_order.return_value)
        self.assertIsInstance(res, HTTPOk)

        # test exception
        req.reset_mock()
        req.registry.notify.side_effect = Exception('boom!')

        res = facebook_payments_status_update_disputed(ctx, req)

        req.registry.notify.assert_called_once_with(m_order.return_value)
        self.assertIsInstance(res, HTTPOk)

    @mock.patch('pyramid_facebook.credits.RefundedOrder')
    def test_facebook_payments_status_update_refunded(self, m_order):
        from pyramid_facebook.credits import facebook_payments_status_update_refunded
        ctx = mock.Mock()
        req = mock.Mock()

        res = facebook_payments_status_update_refunded(ctx, req)

        req.registry.notify.assert_called_once_with(m_order.return_value)
        self.assertIsInstance(res, HTTPOk)

        # test exception
        req.reset_mock()
        req.registry.notify.side_effect = Exception('boom!')

        res = facebook_payments_status_update_refunded(ctx, req)

        req.registry.notify.assert_called_once_with(m_order.return_value)
        self.assertIsInstance(res, HTTPOk)


    @mock.patch('pyramid_facebook.credits.EarnedCurrencyOrder')
    def test_facebook_payments_status_update_placed_currency_app_order(self, m_order):
        from pyramid_facebook.credits import facebook_payments_status_update_placed
        ctx = mock.MagicMock()
        req = mock.Mock()

        res = facebook_payments_status_update_placed(ctx, req)

        req.registry.notify.assert_called_once_with(m_order.return_value)

        expected = {
            'content':{
                'status': 'settled',
                'order_id': ctx.order_details['order_id']
                },
            'method': 'payments_status_update',
            }
        self.assertDictEqual(expected, res)

    @mock.patch('pyramid_facebook.credits.PlacedItemOrder')
    def test_facebook_payments_status_update_placed_item_order(self, m_order):
        from pyramid_facebook.credits import facebook_payments_status_update_placed
        ctx = mock.MagicMock()
        ctx.earned_currency_data = None
        req = mock.Mock()

        facebook_payments_status_update_placed(ctx, req)

        req.registry.notify.assert_called_once_with(m_order.return_value)

    @mock.patch('pyramid_facebook.credits.EarnedCurrencyOrder')
    def test_facebook_payments_status_update_exception(self, m_order):
        from pyramid_facebook.credits import facebook_payments_status_update_placed
        ctx = mock.MagicMock()
        req = mock.Mock()
        req.registry.notify.side_effect = Exception('boooom!')

        res = facebook_payments_status_update_placed(ctx, req)

        req.registry.notify.assert_called_once_with(m_order.return_value)

        expected = {
            'content':{
                'status': 'canceled',
                'order_id': ctx.order_details['order_id']
                },
            'method': 'payments_status_update',
            }
        self.assertDictEqual(expected, res)



# -*- coding: utf-8 -*-
import logging
import urllib

from pyramid.events import NewResponse, subscriber
from pyramid.response import Response
from pyramid.view import view_config
from pyramid.httpexceptions import HTTPForbidden

from pyramid_facebook.events import CanvasRequested
from pyramid_facebook.lib import js_redirect_tpl
from pyramid_facebook.security import ViewCanvas, SignedRequestContext

log = logging.getLogger(__name__)


def includeme(config):
    """Adds routes related to facebook canvas.

    Routes added:

    * ``facebook_canvas`` associated to url ``/{namespace}/`` that musts be
      configured as canvas callback url in facebook application settings.

    * ``facebook_canvas_oauth`` associated to url ``/{namespace}/oauth`` for
      authentication.

    """
    log.debug('Adding route "facebook_canvas".')
    config.add_route(
        'facebook_canvas',
        '/',
        request_method='POST',
        request_param='signed_request',
        factory='pyramid_facebook.security.SignedRequestContext',
        )
    config.scan(package='pyramid_facebook.canvas')


class facebook_canvas(view_config):
    """Decorator that registers a view for the ``facebook_canvas`` route
    with the ``view_canvas`` permission.

    Accepts same arguments as :class:`~pyramid.view.view_config`::

        @facebook_canvas(renderer='canvas.mako')
        def canvas(context, request):
            return {
                'title': 'A great Facebook Game'
                }
    """
    def __init__(self, **kwargs):
        config = kwargs.copy()
        config.update({
            'permission': ViewCanvas,
            'route_name': 'facebook_canvas',
            })
        super(facebook_canvas, self).__init__(**config)


class facebook_install_canvas(view_config):
    """Decorator that registers a view for the ``facebook_canvas`` route
    with context ``~pyramid.httpexceptions.HTTPForbidden``.

    Accepts same arguments as :class:`~pyramid.view.view_config`::

        @facebook_install_canvas(renderer='install.mako')
        def canvas(context, request):
            return {
                'title': 'A great Facebook Game'
                }
    """
    def __init__(self, **kwargs):
        config = kwargs.copy()
        config.update({
            'context': HTTPForbidden,
            'route_name': 'facebook_canvas',
            })
        super(facebook_install_canvas, self).__init__(**config)


@facebook_install_canvas()
def prompt_authorize(context, request):
    """User is not logged in, view prompts user to authorize app:

    * User is not logged in
    * User is on apps.facebook.com/{namespace}
    """
    settings = request.registry.settings
    path = request.route_path('facebook_canvas_oauth')
    query_string = request.GET.copy()
    query_string.pop('code', None)
    redirect_uri = urllib.quote_plus("%s://apps.facebook.com%s?%s" % (
        request.scheme,
        path,
        urllib.urlencode(query_string),
        ))
    url = "%s/dialog/oauth/?client_id=%s&redirect_uri=%s&scope=%s" % (
        "https://www.facebook.com",
        settings['facebook.app_id'],
        redirect_uri,
        settings['facebook.scope'].replace(' ', '')
        )
    log.debug('Prompt authorize, redirecting user on %s', url)
    return Response(js_redirect_tpl % {'location': url})


@facebook_canvas()
def canvas(context, request):
    """When users are logged in, they are authorized to view canvas.

    This view raises :py:exc:`exceptions.NotImplementedError`
    """
    log.error(
        'No facebook canvas configured: use %s decorator.',
        facebook_canvas
        )
    raise NotImplementedError()


@subscriber(
    NewResponse,
    matched_route='facebook_canvas',
    expected_context=SignedRequestContext,
    )
def on_canvas(event):
    try:
        event.request.registry.notify(CanvasRequested(event.request.context,
                                                      event.request))
    except:
        log.debug('CanvasRequested subscriber fail', exc_info=True)



# -*- coding: utf-8 -*-
from pyramid.decorator import reify

from pyramid_facebook.lib import Base


class OauthAccept(Base):
    "Event sent when a user accepts app authentication."


class OauthDeny(Base):
    "Event sent when a user denies app authentication."


class DisputedOrder(Base):
    "Event sent when a user disputes an order."


class RefundedOrder(Base):
    "Event sent when a user got refunded for an order."


class PlacedItemOrder(Base):
    "Event sent when a user placed an item order."


class EarnedCurrencyOrder(Base):
    "Event sent when a user placed an currency order."


class ChangeNotification(Base):
    """Event sent when Facebook notifies about a real-time update.

    For more info, read doc on `Facebook Documentation
    <https://developers.facebook.com/docs/reference/api/realtime/>`_.
    """

    @reify
    def json_body(self):
        """:returns: Request body as a dict. Raised an error if body is json
            decoding fails."""
        return self.request.json_body


class UserSignedRequestParsed(Base):
    """Event sent each time a signed request with user information is
    successfully parsed.
    """


class CanvasRequested(Base):
    """Event sent when an identified user requests the application canvas.
    """



# -*- coding: utf-8 -*-
from __future__ import absolute_import

import unittest
import webtest

from pyramid.config import Configurator
from pyramid.authentication import CallbackAuthenticationPolicy
from pyramid.security import IAuthorizationPolicy, IAuthenticationPolicy

from zope.interface import implementer


@implementer(IAuthenticationPolicy)
class DummyAuthenticationPolicy(CallbackAuthenticationPolicy):
    def unauthenticated_userid(self, request):
        return None


@implementer(IAuthorizationPolicy)
class PermissiveAuthorizationPolicy(object):
    def permits(self, context, principals, permission):
        return True


class TestView(unittest.TestCase):

    _app = None

    _permissive_app = None

    _forbidding_app = None

    @property
    def app(self):
        if TestView._app is None:
            from pyramid_facebook import includeme
            from pyramid_facebook.tests import conf
            config = Configurator(settings=conf)
            config.include(includeme)
            TestView._app = webtest.TestApp(config.make_wsgi_app())
        return TestView._app

    @property
    def permissive_app(self):
        if TestView._permissive_app is None:
            from pyramid_facebook import auth, canvas, credits, real_time, lib
            from pyramid_facebook.tests import conf
            namespace = conf['facebook.namespace']
            config = Configurator(settings=conf)
            config.include(lib)
            config.include(auth, route_prefix=namespace)
            config.include(canvas, route_prefix=namespace)
            config.include(credits, route_prefix=namespace)
            config.include(real_time, route_prefix=namespace)
            config.set_authentication_policy(DummyAuthenticationPolicy())
            config.set_authorization_policy(PermissiveAuthorizationPolicy())
            config.commit()
            TestView._permissive_app = webtest.TestApp(config.make_wsgi_app())
        return TestView._permissive_app



# -*- coding: utf-8 -*-
import unittest

import mock

from facepy import SignedRequest


class TestLib(unittest.TestCase):

    def test_base(self):
        from pyramid_facebook.lib import Base
        req = mock.Mock()
        ctx = mock.Mock()

        b = Base(ctx, req)
        self.assertEqual(ctx, b._context)
        self.assertEqual(req, b._request)
        self.assertEqual(ctx, b.context)
        self.assertEqual(req, b.request)

    def test_encrypt_decrypt_request(self):
        from pyramid_facebook.lib import encrypt_signed_request
        data = {
            u'issued_at': 1297110048,
            u'user': {
                u'locale': u'en_US',
                u'country': u'ca'
                },
            u'algorithm': u'HMAC-SHA256'
            }

        req = encrypt_signed_request('secret key', data)

        res = SignedRequest.parse(req, 'secret key')
        self.assertEqual(data.keys(), res.keys())
        self.assertEqual(data, res)

    def test_request_params_predicate(self):
        from pyramid_facebook.lib import request_params_predicate

        predicate = request_params_predicate('a', 'b', 'c', d=123, e='yo')

        request = mock.Mock()
        request.params = dict(a=1, b=2, c=3, d=123, e='yo')
        self.assertTrue(predicate(None, request))

        request.params = dict(a=1, b=2)
        self.assertFalse(predicate(None, request))

        request.params = dict(a=1, b=2, c=1, d=321, e='oy')
        self.assertFalse(predicate(None, request))

    def test_nor_predicate(self):
        from pyramid_facebook.lib import nor_predicate
        predicate = nor_predicate(test=('a', 'b', 'c'))

        request = mock.Mock()
        request.params = dict(test='a')

        self.assertTrue(predicate(None, request))

        request.params = dict(test='c')
        self.assertTrue(predicate(None, request))

        request.params = dict(not_test='c')
        self.assertFalse(predicate(None, request))

        request.params = dict(test='d')
        self.assertFalse(predicate(None, request))

    def test_headers_predicate(self):
        from pyramid_facebook.lib import headers_predicate

        predicate = headers_predicate('X-header-1', **{'X-header-2': 'value'})

        request = mock.Mock()
        request.headers = {}

        self.assertFalse(predicate(None, request))

        request.headers = {'X-header-1': 'yo'}

        self.assertFalse(predicate(None, request))

        request.headers = {'X-header-1': 'yo', 'X-header-2': 'not value'}

        self.assertFalse(predicate(None, request))

        request.headers = {'X-header-1': 'yo', 'X-header-2': 'value'}

        self.assertTrue(predicate(None, request))


class TestMatchedRoutePredicate(unittest.TestCase):

    def test_predicate_matched(self):
        from pyramid_facebook.lib import MatchedRouteEventPredicate

        pred = MatchedRouteEventPredicate(u'route_name', None)

        event = mock.Mock()
        event.request.matched_route.name = u'route_name'

        self.assertTrue(pred(event))

    def test_predicate_unmatched(self):
        from pyramid_facebook.lib import MatchedRouteEventPredicate

        pred = MatchedRouteEventPredicate(u'route_name', None)

        event = mock.Mock()
        event.request.matched_route.name = u'not_same_route_name'

        self.assertFalse(pred(event))

    def test_text(self):
        from pyramid_facebook.lib import MatchedRouteEventPredicate

        pred = MatchedRouteEventPredicate(u'route_name', None)

        self.assertEqual(u'route_to_match = route_name', pred.text())


class TestContextEcentPredicate(unittest.TestCase):

    def test_predicate_matched(self):
        from pyramid_facebook.lib import ContextEventPredicate
        from pyramid_facebook.security import SignedRequestContext

        event = mock.Mock()
        event.request.context = SignedRequestContext(None)

        pred = ContextEventPredicate(SignedRequestContext, None)

        self.assertTrue(pred(event))

    def test_predicate_unmatched(self):
        from pyramid_facebook.lib import ContextEventPredicate
        from pyramid_facebook.security import (
            SignedRequestContext,
            RealTimeNotificationContext,
            )

        event = mock.Mock()
        event.request.context = RealTimeNotificationContext(None)

        pred = ContextEventPredicate(SignedRequestContext, None)

        self.assertFalse(pred(event))

    def test_text(self):
        from pyramid_facebook.lib import ContextEventPredicate
        from pyramid_facebook.security import SignedRequestContext

        pred = ContextEventPredicate(SignedRequestContext, None)

        self.assertEqual(
            u"expected_context_cls = <class 'pyramid_facebook.security.SignedRequestContext'>",
            pred.text()
            )



# -*- coding: utf-8 -*-
import hashlib
import hmac
import json
import logging

from facepy import FacepyError, GraphAPI, SignedRequest
from facepy.exceptions import SignedRequestError

from pyramid.authorization import ACLAuthorizationPolicy
from pyramid.authentication import CallbackAuthenticationPolicy
from pyramid.decorator import reify
from pyramid.security import Allow

from pyramid_facebook.events import UserSignedRequestParsed

log = logging.getLogger(__name__)

RETRY_NB = 3

ViewCanvas = u'view_canvas'
Authenticate = u'authenticate'
UpdateSubscription = u'update_subscription'
NotifyRealTimeChanges = u'notify_real_time_changes'

FacebookUser = u'facebook-user'
RegisteredUser = u'registered-user'
AdminUser = u'admin-user'
# real time updates https://developers.facebook.com/docs/reference/api/realtime
XHubSigned = u'x-hub-signed'


def includeme(config):
    config.set_authentication_policy(FacebookAuthenticationPolicy())
    config.set_authorization_policy(ACLAuthorizationPolicy())


class SignedRequestContext(object):
    "Security context for facebook signed request routes."

    __acl__ = [
        (Allow, FacebookUser, Authenticate),
        (Allow, RegisteredUser, ViewCanvas),
        ]

    def __init__(self, request):
        self.request = request

    @property
    def facebook_data(self):
        """Contains facebook data provided in ``signed_request`` parameter
        decrypted with :meth:`SignedRequest.parse <facepy.SignedRequest.parse>`
        """
        try:
            return self._facebook_data
        except AttributeError:
            return None

    @facebook_data.setter
    def facebook_data(self, value):
        self._facebook_data = value

    @reify
    def user(self):
        return self.facebook_data['user']

    @reify
    def user_country(self):
        return self.user['country']

    def __repr__(self):
        return '<%s facebook_data=%r>' % (
            self.__class__.__name__,
            self.facebook_data
            )


class FacebookCreditsContext(SignedRequestContext):
    "Context for facebook credits callback requests."

    @reify
    def order_details(self):
        """Order details received in `facebook credits callback for payment
        status updates <http://developers.facebook.com/docs/credits/callback/
        #payments_status_update>`_."""
        return json.loads(
            self.facebook_data['credits']['order_details']
            )

    @reify
    def order_info(self):
        """Order info being the order information passed when the `FB.ui method
        <http://developers.facebook.com/docs/reference/javascript/FB.ui/>`_
        is invoked."""
        return self.facebook_data["credits"]["order_info"]

    @reify
    def earned_currency_data(self):
        """Modified field received in `facebook credits callback for payment
        status update for earned app currency
        <http://developers.facebook.com/docs/credits/callback/
        #payments_status_update_earn_app_currency>`_."""
        data = self.item['data']
        if data:
            try:
                data = json.loads(data)
                data = data['modified'] if 'modified' in data else None
            except:
                data = None
        return data

    @reify
    def item(self):
        """The item info as passed when `FB.ui method
        <http://developers.facebook.com/docs/reference/javascript/FB.ui/>`_
        is invoked."""
        return self.order_details['items'][0]


class AccessTokenContext(object):

    def __init__(self, request):
        self.request = request
        self._user_dict = {}
        self._user_id = None

    @property
    def user_dict(self):
        return self._user_dict

    @user_dict.setter
    def user_dict(self, value):
        self._user_dict = value

    @property
    def user_id(self):
        return self._user_id

    @user_id.setter
    def user_id(self, value):
        self._user_id = value


class AdminContext(AccessTokenContext):
    """Context which defines principals as facebook application id list that
    user administrates.
    """

    def __init__(self, request):
        super(AdminContext, self).__init__(request)
        self.__acl__ = (
            (Allow, request.registry.settings['facebook.app_id'],
             UpdateSubscription),
            )


class RealTimeNotificationContext(object):
    "Context for real-time changes notification route."

    __acl__ = (
        (Allow, XHubSigned, NotifyRealTimeChanges),
        )

    def __init__(self, request):
        self.request = request


class FacebookAuthenticationPolicy(CallbackAuthenticationPolicy):

    unauthenticated_methods = None
    effective_principals_methods = None

    def __init__(self, *args, **kwargs):
        super(FacebookAuthenticationPolicy, self).__init__(*args, **kwargs)

        self.unauthenticated_methods = {
            SignedRequestContext: self._parse_signed_request,
            FacebookCreditsContext: self._parse_signed_request,
            AccessTokenContext: self._validate_access_token,

            RealTimeNotificationContext: lambda req: None,
            }

        self.effective_principals_methods = {
            SignedRequestContext: self._get_principals_from_signed_request,
            FacebookCreditsContext: self._get_principals_from_signed_request,
            AccessTokenContext: self._get_principals_from_access_token,
            RealTimeNotificationContext: self._get_principals_from_signature,
            AdminContext: self._get_principals_from_admin_context,
            }

    def unauthenticated_userid(self, request):
        try:
            method = self.unauthenticated_methods[request.context.__class__]
        except KeyError:
            return None
        else:
            return method(request)

    def effective_principals(self, request):
        cls = FacebookAuthenticationPolicy
        p = super(cls, self).effective_principals(request)
        context = request.context
        try:
            method = self.effective_principals_methods[context.__class__]
        except KeyError:
            pass
        else:
            p.extend(method(request))
        return p

    def remember(self, request, principal, **kwargs):
        return []

    def forget(self, request):
        return []

    def callback(self, user_id, request):
        # by default, having a user id means that user is registered
        return [RegisteredUser]

    def _parse_signed_request(self, request):
        context = request.context
        if 'signed_request' not in request.params:
            return None
        context.facebook_data = None
        try:
            context.facebook_data = SignedRequest.parse(
                request.params[u'signed_request'],
                request.registry.settings[u'facebook.secret_key'],
                )
        except SignedRequestError:
            log.warn(
                'SignedRequestError with signature: %s',
                request.params['signed_request'],
                exc_info=True
                )
            return None
        try:
            user_id = int(context.facebook_data[u'user_id'])
        except KeyError:
            # user_id not in facebook_data => user has not authorized app
            log.debug('User has not authorized app.')
        except ValueError:
            log.warn('Invalid user id %r', context.facebook_data[u'user_id'])
        else:
            request.registry.notify(
                UserSignedRequestParsed(request.context, request)
                )
            return user_id
        return None

    def _validate_access_token(self, request):
        try:
            access_token = request.params['access_token']
        except KeyError:
            return None
        request.context.access_token = access_token
        api = GraphAPI(access_token)
        try:
            request.context.user_dict = api.get('me', retry=RETRY_NB)
        except FacepyError:
            log.warn('Authentication failed', exc_info=True)
        try:
            request.context.user_id = long(request.context.user_dict['id'])
            return request.context.user_id
        except KeyError:
            return None
        except ValueError:
            log.warn('Malformated Facebook response', exc_info=True)
        return None

    def _get_principals_from_signed_request(self, request):
        try:
            if request.context.facebook_data['user_id']:
                return [FacebookUser]
        except:
            pass
        return []

    def _get_principals_from_access_token(self, request):
        p = []
        if request.context.user_id:
            p.append(FacebookUser)
        return p

    def _get_principals_from_admin_context(self, request):
        p = self._get_principals_from_access_token(request)

        try:
            return p + request.context.application_ids
        except AttributeError:
            pass
        # check what apps user owns.
        api = GraphAPI(request.params['access_token'])
        try:
            accounts = api.get('me/accounts', retry=RETRY_NB)
        except FacepyError:
            log.warn('Get accounts failed', exc_info=True)
        else:
            request.context.application_ids = [app['id']
                                               for app in accounts['data']]
            p.extend(request.context.application_ids)
        return p

    def _get_principals_from_signature(self, request):
        # route predicates already check presence of X-Hub-Signature header
        sig = request.headers[u'X-Hub-Signature']
        verif = hmac.new(
            request.registry.settings['facebook.secret_key'],
            request.body,
            hashlib.sha1
            ).hexdigest()
        if sig == ('sha1=%s' % verif):
            return [XHubSigned]
        else:
            log.warn(
                'X-Hub-Signature invalid - expected %s, received %s',
                verif,
                sig,
                )
        return []



# -*- coding: utf-8 -*-
import logging


log = logging.getLogger(__name__)


def includeme(config):
    """Executed when including ``pyramid_facebook``.

    ``pyramid_facebook`` setup:

    * ACL Authorization with :class:`~pyramid.authorization
      .ACLAuthorizationPolicy` using :meth:`config.set_authorization_policy
      <pyramid.config.Configurator.set_authorization_policy>`
    * Authentication with :class:`~pyramid_facebook.security
      .FacebookAuthenticationPolicy` using :meth:`config
      .set_authentication_policy <pyramid.config.Configurator
      .set_authentication_policy>`

    """
    settings = config.registry.settings

    path = '/%s' % settings['facebook.namespace']

    config.include('pyramid_facebook.lib', route_prefix=path)
    config.include('pyramid_facebook.security', route_prefix=path)
    config.include('pyramid_facebook.auth', route_prefix=path)
    config.include('pyramid_facebook.canvas', route_prefix=path)
    config.include('pyramid_facebook.credits', route_prefix=path)
    config.include('pyramid_facebook.real_time', route_prefix=path)
    config.commit()



# -*- coding: utf-8 -*-
import mock

from facepy.exceptions import OAuthError

from pyramid_facebook.tests import conf
from pyramid_facebook.tests.functional import TestView


class TestRealTimeSubscription(TestView):

    def _get_namespace(self):
        return conf['facebook.namespace']

    @mock.patch('pyramid_facebook.real_time.get_application_access_token')
    def test_verify_real_time_subscription(self, m_token):
        m_token.return_value = u'token'

        result = self.app.get(
            '/%s/real-time' % self._get_namespace(),
            {
                'hub.mode': 'subscribe',
                'hub.challenge': u'a challenge to return if success',
                'hub.verify_token': u'token',
                }
            )
        self.assertEqual(u'a challenge to return if success', result.body)

    @mock.patch('pyramid_facebook.real_time.get_application_access_token')
    def test_verify_real_time_subscription_exception(self, m_token):
        m_token.side_effect = OAuthError(code=190, message='Invalid Secret Key')

        self.app.get(
            '/%s/real-time' % self._get_namespace(),
            {
                'hub.mode': 'subscribe',
                'hub.challenge': u'a challenge to return if success',
                'hub.verify_token': u'token',
                },
            status=403,
            )

    @mock.patch('pyramid_facebook.real_time.get_application_access_token')
    def test_verify_real_time_subscription_not_good_token(self, m_token):
        m_token.return_value = u'token'

        self.app.get(
            '/%s/real-time' % self._get_namespace(),
            {
                'hub.mode': 'subscribe',
                'hub.challenge': u'a challenge to return if success',
                'hub.verify_token': u'not the good token',
                },
            status=403,
            )

    @mock.patch('pyramid_facebook.real_time.get_application_access_token')
    @mock.patch('pyramid_facebook.real_time.GraphAPI')
    def test_list_real_time_subscription(self, m_graph_api, m_token):
        m_graph_api.return_value.get.return_value = dict(
            data=[
                dict(
                    active=True,
                    fields=["friends"],
                    object="user",
                    callback_url="http://wow/ww_dev_local_app/real-time"
                    )
                ]
            )
        result = self.permissive_app.get(
            '/%s/real-time/subscriptions' % self._get_namespace(),
            {
                'access_token': 'token'
                }
            )

        self.assertEqual(m_graph_api.return_value.get.return_value, result.json)

    @mock.patch('pyramid_facebook.real_time.get_application_access_token')
    @mock.patch('pyramid_facebook.real_time.GraphAPI')
    def test_list_real_time_subscription_facepy_error(self, m_graph_api, m_token):
        m_graph_api.return_value.get.side_effect = OAuthError(code=190, message='Invalid Secret Key')

        result = self.permissive_app.get(
            '/%s/real-time/subscriptions' % self._get_namespace(),
            {
                'access_token': 'token'
                }
            )
        self.assertEqual(
            {"error": {"message": "Invalid Secret Key", "code": 190, "type": "OAuthError"}},
            result.json
            )

    @mock.patch('pyramid_facebook.real_time.get_application_access_token')
    @mock.patch('pyramid_facebook.real_time.GraphAPI')
    def test_update_real_time_subscription(self, m_graph_api, m_token):
        m_graph_api.return_value.post.return_value = None
        m_token.return_value = 'token'

        result = self.permissive_app.post(
            '/%s/real-time/subscriptions' % self._get_namespace(),
            {
                'access_token': 'token',
                'object': 'user',
                'fields': 'friends,name',
                }
            )
        # facebook returns 'null' which decodes as None
        self.assertIsNone(result.json)

        m_graph_api.return_value.post.assert_called_once_with(
            '%s/subscriptions' % conf['facebook.app_id'],
            object='user',
            fields='friends,name',
            callback_url='http://localhost/%s/real-time' % conf['facebook.namespace'],
            verify_token='token',
            )


    @mock.patch('pyramid_facebook.real_time.get_application_access_token')
    @mock.patch('pyramid_facebook.real_time.GraphAPI')
    def test_update_real_time_subscription_facepy_error(self, m_graph_api, m_token):
        m_graph_api.return_value.post.side_effect = OAuthError(code=190, message='Invalid Secret Key')

        result = self.permissive_app.post(
            '/%s/real-time/subscriptions' % self._get_namespace(),
            {
                'access_token': 'token',
                'object': 'user',
                'fields': 'friends,name',
                }
            )

        self.assertEqual(
            {"error": {"message": "Invalid Secret Key", "code": 190, "type": "OAuthError"}},
            result.json
            )

    @mock.patch('pyramid_facebook.real_time.get_application_access_token')
    @mock.patch('pyramid_facebook.real_time.GraphAPI')
    def test_delete_real_time_subscription(self, m_graph_api, m_token):
        m_graph_api.return_value.delete.return_value = None
        m_token.return_value = 'token'

        result = self.permissive_app.delete(
            '/%s/real-time/subscriptions?access_token=token' % self._get_namespace(),
            )
        # facebook returns 'null' which decodes as None
        self.assertIsNone(result.json)

    @mock.patch('pyramid_facebook.real_time.get_application_access_token')
    @mock.patch('pyramid_facebook.real_time.GraphAPI')
    def test_delete_real_time_subscription_facepy_error(self, m_graph_api, m_token):
        m_graph_api.return_value.delete.side_effect = OAuthError(code=190, message='Invalid Secret Key')

        result = self.permissive_app.delete(
            '/%s/real-time/subscriptions?access_token=token' % self._get_namespace(),
            )

        self.assertEqual(
            {"error": {"message": "Invalid Secret Key", "code": 190, "type": "OAuthError"}},
            result.json
            )

    def test_process_real_time_notification(self):
        headers={'X-Hub-Signature': 'signature', 'Content-Type': 'application/json'}
        result = self.permissive_app.post(
            '/%s/real-time' % self._get_namespace(),
            params='{"object":"user","entry":[{"uid":"1351121254","id":"1351121254","time":1344440243,"changed_fields":["name"]}]}',
            headers=headers,
            )

        self.assertEqual('200 OK', result.status)

    @mock.patch('pyramid_facebook.real_time.ChangeNotification')
    def test_process_real_time_notification_internal_server_error(self, m_event):
        m_event.side_effect = Exception()
        headers={'X-Hub-Signature': 'signature', 'Content-Type': 'application/json'}

        self.permissive_app.post(
            '/%s/real-time' % self._get_namespace(),
            params='{"object":"user","entry":[{"uid":"1351121254","id":"1351121254","time":1344440243,"changed_fields":["name"]}]}',
            headers=headers,
            status=500
            )



# -*- coding: utf-8 -*-
import setuptools

# Distribute *must* be installed (create your virtualenv with --distribute).
if not hasattr(setuptools, "_distribute"):
    raise SystemExit(
        """Setuptools is not supported.
        Please install Distribute (create your virtualenv with --distribute)"""
    )

setup_requires = [
    'd2to1',
    'nose',
    'nosexcover',
    'coverage',
    'mock',
    'webtest',
    'yanc',
    ]

setuptools.setup(
    setup_requires=setup_requires,
    d2to1=True,
    test_suite="nose.collector",
    package_data={'pyramid_facebook': [
        ]},
    paster_plugins=['pyramid'],
    )



# -*- coding: utf-8 -*-
from __future__ import absolute_import

import json

import webtest
from pyramid.config import Configurator

from pyramid_facebook.tests import get_signed_request, conf
from pyramid_facebook.tests.functional import TestView
from pyramid_facebook.credits import facebook_get_currency

# Do not ask why facebook payment callback is that twisted:
# 1 - Read the docs https://developers.facebook.com/docs/credits/callback/
# 2 - if headache take aspirin and go back to point 1

USER_ID = 123
ORDER_ID = 987

credits_data = {
        'buyer': USER_ID,
        'receiver': USER_ID,
        'order_id': ORDER_ID,
        'order_info': '{"item_id":"1"}'
        }

request_data = {
    'user':{
        'country': 'us',
        'locale': 'en_US',
        'age': {
            'min': 21,
            'max': 45
            }
        },
    'user_id': USER_ID,
    'oauth_token':'AAAE0yTNwOxgBABXrDk9tNR0DoWyxwuLLNtpaJCRve8jbUZCr4YbxiIJqdwtHHZBt5uK8wV9ELHFfZBl9DmZAyvcXOuTZC9JZC22BCSfzky1gZDZD',
    'expires': 1325210400,
    'issued_at': 1325203762
    }

order_details = {
    'order_id': ORDER_ID,
    'buyer': USER_ID,
    'app': 9876,
    'receiver': USER_ID,
    'amount': 10,
    'time_placed': 1329243276,
    'update_time': 1329243277,
    'data':'',
    'items':[{
        'item_id': '0',
        'title':'100 Diamonds',
        'description': 'Spend Diamonds in dimonds game.',
        'image_url': 'http://image.com/diamonds.png',
        'product_url':'',
        'price': 10,
        'data':''
        }],
    'status': 'placed'}

earned_app_currency = {
    'modified':{
        'product': 'URL_TO_APP_CURR_WEBPAGE',
        'product_title': 'Diamonds',
        'product_amount': 3,
        'credits_amount':10
        }
    }


def dummy_payments_get_items(context, request):
    return {
        # Required:
        "title":       "100 diamonds",
        "description": "100 shiny diamonds!",
        "price":       1000,
        "image_url":   "http://hadriendavid.com/images/100dimonds.png",

        # Optional (according to facebook doc):
        "item_id": "123",
        "data":    "whatever"
        }

def dummy_payments_get_items_no_title(context, request):
    return {
        # Required:
        "title":       "",
        "description": "100 shiny diamonds!",
        "price":       1000,
        "image_url":   "http://hadriendavid.com/images/100dimonds.png",

        # Optional (according to facebook doc):
        "item_id": "123",
        "data":    "whatever"
        }

@facebook_get_currency()
def dummy_get_currency(context, request):
    name = request.matchdict['name']
    if name == 'TestCoins':
        return {
            'app_id': 123,
            'name': name,
            'description': 'Soft currency for game',
            'image_url': 'http://image.com/diamonds.png',
            'rate': 0.5,
            }
    elif name == 'TestError':
        return 'TestError'

class TestFacebookCredits(TestView):
    """Test based on fb documentation:
    https://developers.facebook.com/docs/credits/callback/"""

    def test_payments_get_items_404(self):
        params = request_data.copy()
        params.update(credits=credits_data)
        params = {
            'signed_request': get_signed_request(**params),
            'method': 'payments_get_items'
            }
        params.update(credits_data)

        self.app.post(
            '/%s/credits' % conf['facebook.namespace'],
            params,
            status=404
            )

    def test_payments_get_items_exception(self):
        from pyramid_facebook import includeme
        from pyramid_facebook.credits import facebook_payments_get_items

        config = Configurator(settings=conf)
        config.include(includeme)
        scanner = type('scanner', (object,) , dict(config=config))
        fbgi = facebook_payments_get_items()
        fbgi._register(scanner, '', dummy_payments_get_items_no_title)

        app = webtest.TestApp(config.make_wsgi_app())

        params = request_data.copy()
        params.update(credits=credits_data)
        params = {
            'signed_request': get_signed_request(**params),
            'method': 'payments_get_items'
            }
        params.update(credits_data)

        with self.assertRaises(ValueError):
            result = app.post(
                '/%s/credits' % conf['facebook.namespace'],
                params,
                )

    def test_payments_get_items(self):
        from pyramid_facebook import includeme
        from pyramid_facebook.credits import facebook_payments_get_items
        from pyramid_facebook.tests import conf
        # to test pyramid integration of facebook_payments_get_items decorator
        # we create a specific config which register dummy_payments_get_items
        # as if it has been decorated with facebook_payments_get_items
        #
        config = Configurator(settings=conf)
        config.include(includeme)
        scanner = type('scanner', (object,) , dict(config=config))
        fbgi = facebook_payments_get_items()
        fbgi._register(scanner, '', dummy_payments_get_items)

        app = webtest.TestApp(config.make_wsgi_app())

        params = request_data.copy()
        params.update(credits=credits_data)
        params = {
            'signed_request': get_signed_request(**params),
            'method': 'payments_get_items'
            }
        params.update(credits_data)
        result = app.post(
            '/%s/credits' % conf['facebook.namespace'],
            params,
            )

        # result musts be a facebook credits compatible json stream
        res_obj = json.loads(result.body)
        expected = {
            'content': [
                {'description': '100 shiny diamonds!',
                'title': '100 diamonds',
                'price': 1000,
                'image_url': 'http://hadriendavid.com/images/100dimonds.png',
                'item_id': '123',
                'data': 'whatever'}
            ],
            'method': 'payments_get_items'
            }
        self.assertEqual(
            expected,
            res_obj
            )

    def test_facebook_get_currency(self):
        from pyramid_facebook import includeme

        config = Configurator(settings=conf)
        config.include(includeme)
        config.scan('pyramid_facebook.tests.functional')

        app = webtest.TestApp(config.make_wsgi_app())

        result = app.get('/currencies/TestCoins', status=200)
        expected_body = """\
                <head prefix="og: http://ogp.me/ns#
                           fb: http://ogp.me/ns/fb#
                           fbpayment: http://ogp.me/ns/fb/fbpayment#">
                      <meta property="fb:app_id" content="123">
                      <meta property="og:type" content="fbpayment:currency">
                      <meta property="og:url" content="http://localhost/currencies/TestCoins">
                      <meta property="og:title" content="TestCoins">
                      <meta property="og:description" content="Soft currency for game">
                      <meta property="og:image" content="http://image.com/diamonds.png">
                      <meta property="fbpayment:rate" content="0.5">
                  </head>
                 """
        self.assertEqual(result.body.split(), expected_body.split())

    def test_payments_get_app_currency_exception(self):
        from pyramid_facebook import includeme

        config = Configurator(settings=conf)
        config.include(includeme)
        config.scan('pyramid_facebook.tests.functional')

        app = webtest.TestApp(config.make_wsgi_app())

        with self.assertRaises(TypeError):
            app.get('/currencies/TestError')

    def test_payments_status_update_placed_item_order(self):
        params = request_data.copy()
        params.update(
            {
            'credits':{
                'order_details': json.dumps(order_details.copy()),
                'status': 'placed',
                'order_id': ORDER_ID
                }
            })

        params = {
            'signed_request': get_signed_request(**params),
            'order_details': json.dumps(order_details.copy()),
            'order_id': ORDER_ID,
            'method': 'payments_status_update',
            'status': 'placed',
            }

        result = self.app.post(
            '/%s/credits' % conf['facebook.namespace'],
            params,
            )

        expected = {
            'content': {
                'status': 'settled',
                'order_id': 987
                },
            'method': 'payments_status_update'
            }
        self.assertDictEqual(expected, json.loads(result.body))


    def test_payments_status_update_earned_app_currency(self):
        params = request_data.copy()
        details = order_details.copy()
        details['items'][0]['data'] = json.dumps(earned_app_currency)
        params.update(
            {
            'credits':{
                'order_details': json.dumps(details),
                'status': 'placed',
                'order_id': ORDER_ID
                }
            })

        params = {
            'signed_request': get_signed_request(**params),
            'order_details': json.dumps(details),
            'order_id': ORDER_ID,
            'method': 'payments_status_update',
            'status': 'placed',
            }

        result = self.app.post(
            '/%s/credits' % conf['facebook.namespace'],
            params,
            )

        expected = {
            'content': {
                'status': 'settled',
                'order_id': 987
                },
            'method': 'payments_status_update'
            }
        self.assertDictEqual(expected, json.loads(result.body))



# -*- coding: utf-8 -*-
import base64
import hashlib
import hmac
import json


js_redirect_tpl = """<html>
  <body>
    <script>
      window.top.location = "%(location)s";
    </script>
  </body>
</html>"""


def includeme(config):
    config.add_subscriber_predicate('matched_route',
                                    MatchedRouteEventPredicate)
    config.add_subscriber_predicate('expected_context', ContextEventPredicate)


class Base(object):
    "Base class for views and events"
    def __init__(self, context, request):
        # request and context as properties to prevent modification
        self._context = context
        self._request = request

    @property
    def context(self):
        """Route context which can be of 2 types:

        * :class:`~pyramid_facebook.security.SignedRequestContext`
        * :class:`~pyramid_facebook.security.FacebookCreditsContext`
        """
        return self._context

    @property
    def request(self):
        "Request object for this route."
        return self._request


def _base64_url_encode(inp):
    """ Facebook base64 decoder based on `Sunil Arora's blog post
    <http://sunilarora.org/parsing-signedrequest-parameter-in-python-bas>`_.

    :param inp: input `str` to be encoded
    :return: `base64` encoded utf8 unicode data
    """
    return unicode(base64.b64encode(inp, '-_').strip('=').encode('utf8'))


def encrypt_signed_request(secret_key, data):
    """Encrypts data the way facebook does for permit testing. Adds algorithm
    key to dict.

    :param secret_key: Facebook application' secret key.
    :param data: a dictionary of data to sign.
    :return: Signed request as defined by `Facebook documentation
             <http://developers.facebook.com/docs/authentication/
             signed_request/>`_

    """
    data = data.copy()
    data.update(algorithm='HMAC-SHA256')

    payload = _base64_url_encode(json.dumps(data)).encode('utf8')
    signature = _base64_url_encode(
        hmac.new(
            secret_key.encode('utf8'),
            msg=payload.encode('utf8'),
            digestmod=hashlib.sha256
            ).digest()
        )
    return '%s.%s' % (signature, payload)


def request_params_predicate(*required_param_keys, **required_params):
    """Custom predicates to check if required parameter are in request
    parameters. Read :ref:`custom route predicates <pyramid:
    custom_route_predicates>`
    for more info::

        # /example?param1&param2=321
        config.add_route(
            'example',
            '/example',
            custom_predicates=[request_params_predicate('param1', param2=321)]
            )
    """
    required = set(required_param_keys)

    def predicate(info, request):
        if not required.issubset(set(request.params.keys())):
            return False
        for k, v in required_params.items():
            if v != request.params.get(k):
                return False
        return True
    return predicate


def nor_predicate(**params):
    """Custom predicate which checks if a parameter is present with possible
    values being one in list values.

    :param params: A dictionary structured as
                   `dict(param_name=(value1, value2))`
    """
    names = set(params.keys())

    def predicate(info, request):
        if not names.issubset(request.params):
            return False
        if [request.params[n]
            for n in names if request.params[n] not in params[n]]:
            return False
        return True
    return predicate


def headers_predicate(*header_names, **headers):
    """Custom predicate which check that `header_names` and  `headers`
    name/value pairs are in `request.headers`.
    """
    def predicate(info, request):
        if [_ for _ in header_names if _ not in request.headers]:
            return False
        if [(k, v) for k, v in headers.iteritems()
            if k not in request.headers or request.headers[k] != v]:
            return False
        return True
    return predicate


class MatchedRouteEventPredicate(object):

    def __init__(self, route_to_match, config):
        self.route_to_match = route_to_match

    def text(self):
        return u'route_to_match = %s' % self.route_to_match

    phash = text

    def __call__(self, event):
        try:
            return event.request.matched_route.name == self.route_to_match
        except:
            return False


class ContextEventPredicate(object):

    def __init__(self, expected_context_cls, config):
        self.expected_context_cls = expected_context_cls

    def text(self):
        return u'expected_context_cls = %r' % self.expected_context_cls

    phash = text

    def __call__(self, event):
        try:
            return isinstance(event.request.context, self.expected_context_cls)
        except:
            return False
